/* -*- C++ -*-
 *
 * testNewCode.cpp
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 * $Id: testAddMapping.cpp,v 1.3 2003/07/23 16:02:55 mika Exp $
 *
 */


#include "EdgReplicaLocationService/ReplicaLocationService.h"
#include "EdgReplicaLocationService/ReplicationExceptions.h"
#include <vector>

#include <time.h>

#ifdef __linux__
#include <sys/time.h>
#endif


typedef long long TimeT;
// return nanosec since the Epoc
inline TimeT  universalTime() { 
#ifdef __linux__
  timeval now; 
  gettimeofday(&now,0);
  return 1000000000LL*now.tv_sec + 1000LL* now.tv_usec;
#else
  timespec now;
  clock_gettime(CLOCK_REALTIME,&now);
  return 1000000000LL*now.tv_sec + now.tv_nsec;
#endif
}


#ifdef WITH_WP2_SECURITY
#include <WP2Security.h>
// Defs for getopt
char optchar, *optarg;
int optind, optopt;
int opterr = 1;

void usage(char *);

#endif

int main( int argc, char * const argv[]) 
{
  using namespace EdgReplicaLocationService;
  
  LocalReplicaCatalog *lrc = NULL;

  // modif. Maria 
  int ninserts=10;
  int wtime=0;

#ifdef WITH_WP2_SECURITY
  // Let's see what args were given
  char *credentials = NULL;
  char *passphrase = NULL;
  char *cacertdir = NULL;
  char *rseedfile = NULL;
  int use_security = 0;
  char *role = NULL;
  char *policy = NULL;

  char *accessurl = "https://lxshare0201.cern.ch:8443/edg-replica-location/services/edg-local-replica-catalog";

  if (argc > 0) {
    while ((optchar = getopt(argc, argv, "c:p:d:r:u:R:P:nhv")) > 0) {
      switch (optchar) {
      case 'c':
	// File containing user's public credentials (cert+key)
	credentials = optarg;
	break;
      case 'p':
	// User's private passphrase
	passphrase = optarg;
	break;
      case 'd':
	// Path to the dir of trusted CA certs
	cacertdir = optarg;
	break;
      case 'r':
	// Random seed file
	rseedfile = optarg;
	break;
      case 'u':
	// Access url defining the service
	accessurl = optarg;
	break;
      case 'R':
	// Define role
	role = optarg;
	break;
      case 'P':
	// Define policy
	policy = optarg;
	break;
      case 'w':
	wtime = atoi(optarg);
	break;
      case 'n':
	ninserts = atoi(optarg);
	if (ninserts > 999999) {
	  usage(argv[0]);
	  exit(1);
	}
	break;
      default:
      case 'v':
      case 'h':
	usage(argv[0]);
	exit(1);
      }
    }
  }
  
  use_security = (!strncmp("https:",accessurl,6)) ? 1 : 0;

  const std::string theBoltedUrl(accessurl);

  if (use_security)
    lrc = ReplicaLocationService::getLocalReplicaCatalog(theBoltedUrl,
							 passphrase,
							 credentials,
							 cacertdir,
							 rseedfile,
							 role,
							 policy,
							 "myDefaultRole",
							 "myDefaultPolicy");
  else
    lrc = ReplicaLocationService::getLocalReplicaCatalog(theBoltedUrl);

#else
  
    if(argc<4) {
      cout << "arguments:  -w wait time(second), -n number of inserts " << endl;
      exit(-1);
    }

    lrc = ReplicaLocationService::getLocalReplicaCatalog("http://lxshare0201.cern.ch:8080/edg-replica-location/services/edg-local-replica-catalog");
  
    for (int ix=1;ix<argc;++ix) {
      char *pchar=argv[ix];
      switch (pchar[0]) {
      case '-':
	switch(pchar[1]) {
	case 'w':
	  wtime=atoi(argv[ix+1]);
	  break;
	case 'n':
	  ninserts=atoi(argv[ix+1]);
	  break;
	}//end of switch on pchar[0]
      }//end of switch on pchar[1]
    }//end of loop on arguments

    if ( ninserts > 999999 ) {
      cout << "-n "<<ninserts<<"are too many insertion for this job.";
      cout << " The maximum allow is 999999"<<endl;
    }

#endif

    const std::string guid("guid:1");

      // Create a new entry 
      // Modif. Maria: loop for creating ninserts 
  
    try {

      char filename[40]; 
      char hostname[20];
      char *lfn;
      if(gethostname(hostname,20)){cout << "failed to get host name\n"; }
  
      int procid=getpid();
  
      for (int index=1; index<=ninserts ;++index) {
	sprintf(filename,"lfn_%s_%06d%06d",hostname,procid,index);
	cout << index << " : "<< filename <<endl;
	//       lfn = "lfn0";
	lfn = filename;   
	if ( index == 1 ) {
	  cout << "Creating a first entry in RLS: "<<
	    lfn << ' ' << guid << endl;
	}
	if ( index == ninserts ) {
	  cout << "Creating the last entry in RLS: "<< 
	    lfn << ' ' << guid << endl;
	}
	lrc->addMapping(guid, lfn);
      }
      
    }catch(exception& e) {
      cout << "caught exception: " << e.what() << endl;
    } catch(... ) {
      cout << "Caught exception (which was not child of exception)" << endl;
    }
  delete lrc;

  return 0;
} 

#ifdef WITH_WP2_SECURITY

void usage(char *pname) {
  cerr << "Usage: " << pname << " [ options ]" << endl;
  cerr << "This product includes software developed by the OpenSSL ";
  cerr << "Project\nfor use in the OpenSSL Toolkit. (http://www.openssl.org/)";
  cerr << endl << endl << "Options:" << endl;
  cerr << "-c your_public_cert+key_file" << endl;
  cerr << "-d dir_of_trusted_CA_certs" << endl;
  cerr << "-p your_private_keys_passphrase" << endl;
  cerr << "-r path_to_random_seed_file" << endl;
  cerr << "-R role" << endl;
  cerr << "-P policy" << endl;
  cerr << "-u service_url" << endl;
  cerr << "-n number_of_inserts (max 999999)" << endl;
  cerr << "-w wait_time" << endl;
  cerr << "-h (prints help and version number)" << endl;
  cerr << "-v (prints help and version number)" << endl;
  cerr << endl << "Defaults:" << endl;
  cerr << "c = /tmp/x509up_uUID (i.e. your user proxy cert)" << endl;
  cerr << "d = /etc/grid-security/certificates" << endl;
  cerr << "r = ~/.rnd" << endl;
  cerr << "u = (a hard coded url in the sample application)" << endl;
  cerr << "n = 10" << endl;
  cerr << "w = 0" << endl;
  cerr << "R = myDefaultRole" << endl;
  cerr << "P = myDefaultPolicy" << endl << endl;
  cerr << "It's better to run grid-proxy-init than using long term " << endl;
  cerr << "credentials with -c and -p." << endl;
}

#endif
